/*
 *  Command.cpp
 *  godivacli
 *
 *  Created by Claudio Bisegni on 25/09/10.
 *  Copyright 2010 INFN. All rights reserved.
 *
 */

#include "Command.h"
#include <sstream>
#include <stdlib.h>
static struct Namespace testNS[] =
{
	{"SOAP-ENV", "http://schemas.xmlsoap.org/soap/envelope/"},    /* MUST be first */
	{"SOAP-ENC", "http://schemas.xmlsoap.org/soap/encoding/"},    /* MUST be second */
	{"xsi", "http://www.w3.org/1999/XMLSchema-instance"},         /* MUST be third */
	{"xsd", "http://www.w3.org/1999/XMLSchema"},
	{"ns1", "http://slfservletadaptor.server.ref.org/"},       /* Method namespace URI */
	{NULL, NULL}
};

std::string  Command::clientCertPath;
std::string  Command::clientCertPwd;
RefWSApiCallInterfacePortBindingProxy Command::service;
bool  Command::soapServiceInitialized = false;
bool  Command::printSerialization = false;

/*!
    @method     Command
    @abstract   Costruttore
*/
Command::Command(const char* gName, const char* aName){
	apiGroup = new std::string(gName);
	apiName = new std::string(aName);
	apiResult = new ns1__refwsCallResult();
	soap = new struct soap;
	if(!soapServiceInitialized){
		if(soap) soap_init(soap); 

		if (soap_ssl_client_context(&service,
									SOAP_SSL_SKIP_HOST_CHECK|SOAP_SSL_ALLOW_EXPIRED_CERTIFICATE|SOAP_SSL_REQUIRE_CLIENT_AUTHENTICATION, /* use SOAP_SSL_DEFAULT in production code */
									clientCertPath.size()>0?clientCertPath.c_str():NULL,       /* keyfile: required only when client must authenticate to 
												 server (see SSL docs on how to obtain this file) */
									clientCertPwd.size()>0?clientCertPwd.c_str():NULL,       /* password to read the keyfile */
									NULL,      /* optional cacert file to store trusted certificates */
									NULL,      /* optional capath to directory with trusted certificates */
									NULL      /* if randfile!=NULL: use a file with random data to seed randomness */ 
									))
		{ 
			soap_print_fault(&service, stderr);
		}
		//cleaer the used password
		clientCertPwd.clear();
		soap_set_namespaces(&service, testNS); 
		soapServiceInitialized = true;
	}
}

/*!
 @method     Command
 @abstract   Costruttore
 */
Command::~Command(){
	if(apiName) delete(apiName);
	if(apiGroup) delete(apiGroup);
	if(apiResult) delete (apiResult);
	if(soap) delete(soap);
}

/*!
 @method     setEndPoint
 @abstract   Costruttore
 */
void Command::setEndPoint(const char *newEndPoint) {
	service.soap_endpoint = newEndPoint;
}

/*!
 @method     inError
 @abstract   Rtorna true se 'e stato ritornato un errore
 */
bool Command::inError() {
	return apiResult?apiResult->errorDescription!=NULL:false;
}

/*!
 @method     isMultipleResult
 @abstract   Ritorna true se il risultato e' di tipo multiplo
 */
bool Command::isMultipleResult() {
    return apiResult->listResult.size() >0;
}

/*!
 @method     error
 @abstract   Costruttore
 */
const char * Command::getErrorString() {
	if(!apiResult) return NULL;
	return apiResult->errorDescription->c_str();
}

/*!
 @method     result
 @abstract   Ritorna il risultato dell'operazione chiamata
 */
xsd__anyType* Command::getResult() {
	if(!apiResult) return NULL;
    
    if(apiResult->result){
        return apiResult->result;
    } else return NULL;
}

/*!
 @method     getMultipleResult
 @abstract   Ritorna un vettore di risultati dell'operazione chiamata
 */
std::vector<xsd__anyType*>  *Command::getMultipleResult(){
    if(!apiResult) return NULL;
    
    if(apiResult->listResult.size() >0){
        return &apiResult->listResult;
    } else return NULL;
}

/*!
 @method     call
 @abstract   Costruttore
 */
void  Command::call() {
	int error = 0;
	//godiva request class
	ns1__executeApi request;
	//godiva response class
	ns1__executeApiResponse response;

	ns1__refwsCallParam apiParam;
	
	apiParam.webServicesParam = paramToSerialize;
	
	
	request.refApiGroupName = apiGroup;
	request.refApiName = apiName;
	request.refApiParamsXmlSerialized = serializeParam(&apiParam);
	if(request.refApiParamsXmlSerialized && Command::printSerialization){
		std::cout << "Param Serialized:\n" << request.refApiParamsXmlSerialized->c_str()  << std::endl; 		
	}
    
	if ((error = service.executeApi(&request, &response)) == SOAP_OK){
		deserializeParam(response.refApiSerializedOutput);
	} else {
		soap_print_fault(&service, stderr);
	}

	
	if(request.refApiParamsXmlSerialized)delete(request.refApiParamsXmlSerialized);
}

/*!
 @method     
 @abstract   <#(brief description)#>
 @discussion <#(comprehensive description)#>
 */
void Command::setApiResult(ns1__refwsCallResult *newApiResult) {
	if(apiResult){
		delete(apiResult);
	}
	if(newApiResult)
		apiResult = newApiResult;
}

//------------------action---------------------
/*!
 @method     
 @abstract   <#(brief description)#>
 @discussion <#(comprehensive description)#>
 */
void Command::addServiceParam(xsd__anyType* param) {
	paramToSerialize.push_back(param);
}

/*!
 @method     
 @abstract   <#(brief description)#>
 @discussion <#(comprehensive description)#>
 */
void Command::clearParam() {
	paramToSerialize.clear();
}

//-------------------------------------------------------------------------------
/*!
 @method     
 @abstract   <#(brief description)#>
 @discussion <#(comprehensive description)#>
 */
std::string* Command::serializeParam(ns1__refwsCallParam *param) {
	std::ostringstream ostream;
	if(!param || param->webServicesParam.size()==0) return NULL;
	
	soap_destroy(soap); // remove deserialized C++ objects 
	//soap_omode(&soap, SOAP_ENC_ZLIB | SOAP_XML_DEFAULTNS|SOAP_XML_IGNORENS); 
	soap_begin(soap); 
	soap_begin_send(soap); 
	param->soap_serialize(soap);
	(*soap).os = &ostream;
	param->soap_put(soap, "REFWSCallParam", NULL); 
	soap_end_send(soap); 
	soap_destroy(soap); 
	soap_end(soap); 
	soap_done(soap); 

	return new std::string(ostream.str());
}

/*!
 @method     
 @abstract   <#(brief description)#>
 @discussion <#(comprehensive description)#>
 */
void Command::deserializeParam(std::string *serializedParam) {
	if(!serializedParam) return;
	
	if(serializedParam && Command::printSerialization){
		std::cout << "Result serialized:\n" << serializedParam->c_str()  << std::endl; 		
	}
	
	
	std::istringstream serializedOutputStream(serializedParam->c_str());

	soap_init(soap); 
	//soap_omode(&soap, SOAP_XML_DEFAULTNS|SOAP_XML_IGNORENS); 
	soap_begin(soap); 
	(*soap).is = &serializedOutputStream;
	soap_begin_recv(soap); 
	soap_get_ns1__refwsCallResult(soap, apiResult, NULL, NULL);

	
	soap_end_recv(soap); // check consistency of id/hrefs 

	soap_end(soap); // remove deserialized data 
	soap_done(soap); // finalize last use of the context
}
