//
//  MainController.cp
//  godivacli
//
//  Created by Claudio Bisegni on 16/10/10.
//  Copyright (c) 2010 INFN. All rights reserved.
//
#include <iostream>
#include <cstdio>
#include "soapRefWSApiCallInterfacePortBindingProxy.h" 
#include "RefWSApiCallInterfacePortBinding.nsmap" 
#include "MainController.h"
#include "UUIDfromCFCommand.h"
#include "SyncFromProtoserv.h"
#include "GetUsernameFromNameSurname.h"
#include "Authentication.h"
#include "MainController.h"
#include "GodivaCliOutput.h"

using namespace std;
using namespace TCLAP;
/*!
 @method     <#Nome Metodo#>
 @abstract   <#Descrizione metodo#>
 */
MainController::MainController(int argCount, char** argValue){
    argc = argCount;
    argv = argValue;
	
	cmd = new CmdLine("GODiVA Commandline Utility", ' ', "0.9.1");
    ufcArg = new ValueArg<string> ("","ufc","Return the UUID associated to identity by Codice Fiscale",false,"codice fiscale","string");
    
	syncFromProtoservArg  = new ValueArg<string> ("","sfp","Sync the identity identified by Codice Fiscale from protoser into AAI",false,"codice fiscale","string");
	
    uidFromNameSurnameSwitch=new SwitchArg("", "username", "Return the available username for name and surname(to use with --name and --surname param)", false);
    ufnsName = new ValueArg<string>("","name","The name for ufns command",false,"Name","string");
    ufnsSurname = new ValueArg<string>("","surname","The surname for ufns command",false,"Surname","string");
    
    noSSLSwitch = new SwitchArg("", "nossl", "Set the comunication on normal httpstream", false);
    hostArg = new ValueArg<string>("","host","Set the host where to connect",false,"host.domain","string");
    portArg = new ValueArg<int>("","port","Set the tcp port where to connect",false,8080,"int");
    
    clientCertArg = new ValueArg<string>("","cert","Set the client pem certificate file",false,"/path/client.pem","string");
    clientCertPwdArg = new ValueArg<string>("","certpwd","Set the password for client pem certificate file ",false,"secret","string");
    
	
    
    printSerializeXMLArg = new SwitchArg("", "print_serialization", "print on the output stream the xml serialization for param sent and result received", false);
	interactiveModeArg = new SwitchArg("", "interactive", "Interactive mode", false);
	quitInteractiveModeArg = new SwitchArg("", "quit", "Quit Interactive mode", false);
}
/*!
 @method     <#Nome Metodo#>
 @abstract   <#Descrizione metodo#>
 */
MainController::~MainController(){
	if(cmd)delete(cmd);
    if(ufcArg)delete(ufcArg);
	if(syncFromProtoservArg)delete(syncFromProtoservArg);
    if(uidFromNameSurnameSwitch)delete(uidFromNameSurnameSwitch);
    if(ufnsName)delete(ufnsName);
    if(ufnsSurname)delete(ufnsSurname);
    if(noSSLSwitch)delete(noSSLSwitch);
    if(hostArg)delete(hostArg);
    if(portArg)delete(portArg);
    if(clientCertArg)delete(clientCertArg);
    if(clientCertPwdArg)delete(clientCertPwdArg);
    if(printSerializeXMLArg)delete(printSerializeXMLArg);
	if(interactiveModeArg)delete(interactiveModeArg);
	if(quitInteractiveModeArg)delete(quitInteractiveModeArg);
}

/*!
 @method     <#Nome Metodo#>
 @abstract   <#Descrizione metodo#>
 */
void MainController::start() {
	try {
		cmd->add(*ufcArg);
		cmd->add(*syncFromProtoservArg);
		cmd->add(*hostArg);
		cmd->add(*portArg);
		cmd->add(*uidFromNameSurnameSwitch);
		cmd->add(*ufnsName);
		cmd->add(*ufnsSurname);
		cmd->add(*clientCertArg);
		cmd->add(*clientCertPwdArg);
		cmd->add(*interactiveModeArg);
		cmd->add(*quitInteractiveModeArg);
		cmd->add(*noSSLSwitch);
		cmd->parse( argc, argv );
		
		if(noSSLSwitch->isSet()){
		 soap_ssl_init();
		}
		
		//create the server URL
		composeServerURL();
		
		//fill constant
		if(interactiveModeArg && interactiveModeArg->isSet()){
			//execute interactive mode
			interactiveMode();
		}else {
			//execute standard single operation
			checkAndExecuteCommand();
		}
	} catch (TCLAP::ArgException &e) { 
		std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl; 
	}
}

/*!
 @method     fillCommandConstants
 @abstract   <#Descrizione metodo#>
 */
bool MainController::checkAndExecuteCommand(){
	//serialization print output
	Command::setClientPrintSerialization(printSerializeXMLArg->isSet());
	//check for client cert file
	if(clientCertArg->isSet()){
		Command::setClientCertPath(&clientCertArg->getValue());
	}
	//check cert passwrod for cert file
	if(clientCertPwdArg->isSet()){
		Command::setClientCertPassword(&clientCertPwdArg->getValue());
	}
	
	//
	if(clientCertArg->isSet() && clientCertPwdArg->isSet()){
		executeAuthentication();
	}

	if (ufcArg->isSet()) {
		executeUFC(ufcArg->getValue().c_str());
	}else if (uidFromNameSurnameSwitch->isSet()) {
		if(ufnsName->isSet() && ufnsSurname->isSet()){
			executeUidFromNameSurname(ufnsName->getValue().c_str(), ufnsSurname->getValue().c_str());				
		}else{
			std::cerr << "error: the --username command is to be used with --name and --surname comands"<<  std::endl; 
		}
	}else if (syncFromProtoservArg->isSet()) {
		executeSFP(syncFromProtoservArg->getValue().c_str());
	}
	
	return quitInteractiveModeArg && quitInteractiveModeArg->isSet();
}

/*!
 @method     fillCommandConstants
 @abstract   <#Descrizione metodo#>
 */
void MainController::interactiveMode(){
	GodivaCliOutput customOutput;
	bool endInteractive = false;
	vector<string> userInputCommandTokens;
	
	//clear the terminal window
	system("clear");
	
	//set the custo output class
	cmd->setOutput(&customOutput);
	while(!endInteractive){
		cmd->reset();
		userInputCommandTokens.clear();
		userInputCommandTokens.push_back(argv[0]);
		cout << GODIVA_PROMPT;
		getUserCommand(userInputCommandTokens);
		try {
			cmd->parse(userInputCommandTokens);
		}
		catch (TCLAP::ArgException &e) {
			std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl; 
		}
		
		endInteractive = checkAndExecuteCommand();
	}
}

/*!
 @method     <#Nome Metodo#>
 @abstract   <#Descrizione metodo#>
 */
void MainController::getUserCommand(vector<string> &parsedCommandTokens) {
    string userInput;
    string inputDelimiter = " ";
    getline(cin, userInput);
    tokenizeSentence(userInput, parsedCommandTokens, inputDelimiter);
}

/*!
 @method     tokenizeSentence
 @abstract   divide the string in input into tokens
 */
void MainController::tokenizeSentence(const string& str, vector<string>& tokens, const string& delimiters) {
        // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
        // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);
    
    while (string::npos != pos || string::npos != lastPos) {
            // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
            // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
            // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}

/*!
 @method     executeUFC
 @abstract   
 @discussion 
 */
void MainController::composeServerURL() {
	serverURL.clear();
	if(noSSLSwitch && noSSLSwitch->isSet()){
		serverURL.append(PROTOCOLL_NOSSL);
	}else {
		serverURL.append(PROTOCOLL_SSL);
	}
	
	
	if(hostArg && hostArg->isSet()){
		serverURL.append(hostArg->getValue());
	}else{
		serverURL.append(HOST);
	}
	
	serverURL.append(":");
	stringstream portString;
	if(portArg && portArg->isSet()){
		portString << portArg->getValue();
	}else{
		portString << PORT;
	}
	
	serverURL.append(portString.str());
	serverURL.append("/GODiVA/RefWSApiCallInterfaceService");
}


/*!
 @method     executeAuthentication
 @abstract   Esegue l'authenticazione usando le informazioni contenute nel certificato,
 tale oeprazione viene eseguita interamente lato server
 */
void MainController::executeAuthentication() {
    Authentication authAction;
	authAction.setEndPoint(serverURL.c_str());
    authAction.call();
}

/*!
 @method     executeUFC
 @abstract   
 @discussion 
 */
void MainController::executeUFC(const char *cf) {
	UUIDfromCFCommand uuidFroCF;
	uuidFroCF.setEndPoint(serverURL.c_str());
	uuidFroCF.setCodiceFiscale(cf);
	uuidFroCF.call();
	if(uuidFroCF.inError()){
		std::cout << "error: " << uuidFroCF.getErrorString() <<  std::endl; 
	}else if (uuidFroCF.getResult()) {
		std::cout << "uuid: " << ((xsd__string*)uuidFroCF.getResult())->__item <<  std::endl; 
	}else{
		std::cout << "no error and no result" <<  std::endl; 
	}
}

/*!
 @method     executeUFC
 @abstract   
 @discussion 
 */
void MainController::executeSFP(const char *cf) {
	SyncFromProtoserv syncFromProtoserv;
	syncFromProtoserv.setEndPoint(serverURL.c_str());
	syncFromProtoserv.setCodiceFiscale(cf);
	syncFromProtoserv.call();
	if(syncFromProtoserv.inError()){
		std::cout << "error: " << syncFromProtoserv.getErrorString() <<  std::endl; 
	}else if (syncFromProtoserv.getResult()) {
		std::cout << "Protoserv Sync Log:" << 
		std::endl << 
		((xsd__string*)syncFromProtoserv.getResult())->__item <<  
		std::endl; 
	}else{
		std::cout << "no error and no result" <<  std::endl; 
	}
}

/*!
 @method     executeUFC
 @abstract   
 @discussion 
 */
void MainController::executeUidFromNameSurname(const char *name, const char *surname) {
	GetUsernameFromNameSurname uidFromNS;
	uidFromNS.setEndPoint(serverURL.c_str());
	uidFromNS.setName(name);
	uidFromNS.setSurname(surname);
	uidFromNS.call();
	if(uidFromNS.inError()){
		std::cout << "error: " << uidFromNS.getErrorString() <<  std::endl; 
	}else if (uidFromNS.getResult()) {
		std::cout << "uuid: " << ((xsd__string*)uidFromNS.getResult())->__item <<  std::endl; 
	}else{
		std::cout << "no error and no result" <<  std::endl; 
	}
}
